########################################
#
# funzioni che definiscono il problema
# 
#      min  f(x)
#      s.t. g(x) <= 0
#           h(x) == 0
# 
# quando:
#
# f(x) = x_1 + x_2
# g_1(x) = x_1^2+x_2^2 - 2
# g_2(x) = -x_2
# h(x) vettore vuoto
# 
########################################
module problema
	#export f, g, h
	#export n, m, p
	export collection
	export show
	export use
	export find

	type prob_obj
		name	:: String
		n		:: Int
		m		:: Int
		p		:: Int
		desc	:: String
		f		:: Function
		g		:: Function
		h		:: Function
		x0		:: Array
	end

	collection = Dict{Int,prob_obj}()

"""
``find(c,str)``

where c is a dictionary of prob_obj objects and str is a String.
Returns the prob_obj in dictionary c such that c.name == nome. If that does not exist, returns nothing
"""
function Base.find(c::Dict{Int,prob_obj},nome::String)
	for (i,p) in c
	#for i = 1:c.count
		if p.name == nome
			return p
		end
	end		
	return nothing
end

	function use(prob::prob_obj)
		return (prob.n,prob.m,prob.p,prob.f,prob.g,prob.h,prob.x0)
	end
	function use(ind::Integer)
		if ind <= 0
			return nothing
		else
			prob = collection[ind]
			return (prob.n,prob.m,prob.p,prob.f,prob.g,prob.h,prob.x0)
		end
	end

	collection[1] = prob_obj("ex1",2,2,0,
		      "min x[1]+x[2]\ns.t. x[1]^2+x[2]^2-2.0 <= 0\n     x[2] >= 0",
		      x -> x[1]+x[2],
		      x -> [
		           (x[1]^2+x[2]^2-2.0),
			   -x[2]
			   ],
		      x -> zeros(0),
			  [[2;2], [-2;-2]])
	collection[2] = prob_obj("ex2",2,2,0,
		      "min (x[1]+3)^2+(x[2]+3)^2\ns.t. x[1]^2+x[2]^2-2.0 <= 0\n     x[2] >= 0",
		      x -> (x[1]+3)^2+(x[2]+3)^2,
		      x -> [
		      	   (x[1]^2+x[2]^2-2.0),
		      	   -x[2]
			   ],
		      x -> zeros(0),
			  [[-1.2; 1]])
	collection[3] = prob_obj("hs06",2,0,1,
		      "min (1-x[1])^2\ns.t. 10*(x[2]-x[1]^2) = 0",
		      x -> (1-x[1])^2,
		      x -> zeros(0),
		      x -> [
		      	   10*(x[2]-x[1]^2)
		      	   ],
			  [[-1.2; 1]])
	collection[4] = prob_obj("hs07",2,0,1,
		      "min log(1+x[1]^2) - x[2]\ns.t. (1+x[1]^2)^2 + x[2]^2 = 4",
		      x -> log(1+x[1]^2) - x[2],
		      x -> zeros(0),
		      x -> [
		      	   ((1+x[1]^2)^2 + x[2]^2 - 4)
		      	   ],
			  [[2; 2]])
	collection[5] = prob_obj("hs08",2,0,2,
		      "min -1\ns.t. x[1]^2 + x[2]^2 = 25\n     x[1]x[2] = 9",
		      x -> -1,
		      x -> zeros(0),
		      x -> [
		      	   (x[1]^2 + x[2]^2 - 25),
			   (x[1]x[2] - 9)
		      	   ],
			  [[2; 1]])
	collection[6] = prob_obj("hs09",2,0,1,
		      "min sin(pi*x[1]/12)cos(pi*x[2]/16)\ns.t. 4x[1] - 3x[2] = 0",
		      x -> sin(pi*x[1]/12)cos(pi*x[2]/16),
		      x -> zeros(0),
		      x -> [
		      	   (4x[1] - 3x[2])
		      	   ],
			  [[0; 0]])
	collection[7] = prob_obj("hs10",2,1,0,
		      "min x[1]-x[2]\ns.t. 3x[1]^2 - 2x[1]x[2] + x[2]^2 <= 1",
		      x -> x[1]-x[2],
		      x -> [(3x[1]^2 - 2x[1]x[2] + x[2]^2 -1)],
		      x -> zeros(0),
			  [[-5; 5]])	
	collection[8] = prob_obj("hs11",2,1,0,
		      "min (x[1]-5)^2+x[2]^2-25\ns.t. x[1]^2 - x[2] <= 0",
		      x -> (x[1]-5)^2+x[2]^2-25,
		      x -> [(x[1]^2 - x[2])],
		      x -> zeros(0),
			  [[4.9; 0.1]])	
	collection[9] = prob_obj("hs12",2,1,0,
		      "min 0.5x[1]^2 + x[2]^2 -x[1]x[2] - 7x[1] - 7x[2]\ns.t. 4x[1]^2 + x[2]^2 <= 25",
		      x -> 0.5x[1]^2 + x[2]^2 -x[1]x[2] - 7x[1] - 7x[2],
		      x -> [(4x[1]^2 + x[2]^2 - 25)],
		      x -> zeros(0),
			  [[0; 0]])	
	collection[10]= prob_obj("hs13",2,3,0,
		      "min (x[1]-2)^2 + x[2]^2\ns.t. -(1-x[1])^3 + x[2] <= 0\n     x[1], x[2] >= 0",
		      x -> (x[1]-2)^2 + x[2]^2,
		      x -> [(-(1-x[1])^3 + x[2]),
		      	   -x[1],
			   -x[2]
			   ],
		      x -> zeros(0),
			  [[-2; -2]])	
	collection[11]= prob_obj("ex3",2,1,0,
		      "min -0.1(x[1]-4)^2 + x[2]^2\ns.t. x[1]^2+x[2]^2 >= 1",
		      x -> -0.1(x[1]-4)^2 + x[2]^2,
		      x -> [(-x[1]^2-x[2]^2 + 1)],
		      x -> zeros(0),
			  [[1; 1]])	
	collection[12]= prob_obj("ex4",1,1,0,
		      "min x[1]^2\ns.t. x[1] >= 1",
		      x -> x[1]^2,
		      x -> [(1-x[1])],
		      x -> zeros(0),
			  [[-1]])	
	collection[13]= prob_obj("ex5",1,0,1,
		      "min -x[1]^4\ns.t. x[1] = 1",
		      x -> -x[1]^4,
		      x -> zeros(0),
		      x -> [(x[1]-1)],
			  [[-1]])	
	collection[14]= prob_obj("ex6",2,0,1,
		      "min -x[1]-x[2]\ns.t. x[1]^2+x[2]^2 = 1",
		      x -> -x[1]-x[2],
		      x -> zeros(0),
		      x -> [(x[1]^2+x[2]^2 - 1)],
			  [[-1.2; 1]])	
	collection[15]= prob_obj("ex7",2,1,0,
		      "min -x[1]-x[2]\ns.t. x[1]^2+x[2]^2 <= 1",
		      x -> -x[1]-x[2],
		      x -> [(x[1]^2+x[2]^2 - 1)],
		      x -> zeros(0),
			  [[0; 0]])	
	collection[16]= prob_obj("maratos",2,0,1,
		      "min -x[1] + 2(x[1]^2+x[2]^2-1)\ns.t. x[1]^2+x[2]^2 = 1",
		      x -> -x[1] + 2(x[1]^2+x[2]^2-1),
		      x -> zeros(0),
		      x -> [(x[1]^2+x[2]^2 - 1)],
			  [[-1.2; 1]])	
	collection[17]= prob_obj("hs14",2,1,1,
		      "min (x[1]-2)^2+(x[2]-1)^2\ns.t. x[1]^2/4+x[2]^2 <= 1\n     x[1]-2x[2] = -1",
		      x -> (x[1]-2)^2+(x[2]-1)^2,
		      x -> [(x[1]^2/4+x[2]^2 - 1)],
		      x -> [(x[1]-2x[2] + 1)],
			  [[2; 2]])	
	collection[18]= prob_obj("hs15",2,3,0,
			  "min 100(x[2]-x[1]^2)^2+(1-x[1])^2\ns.t. x[1]x[2] - 1 >= 0\n     x[1]+x[2]^2 >= 0\n     x[1] <= 0.5",
			  x -> 100(x[2]-x[1]^2)^2+(1-x[1])^2,
			  x -> [(-x[1]x[2] + 1),
				    (-x[1]-x[2]^2), (x[1]-0.5)
				   ],
			  x -> zeros(0),
			  [[-2; 1]])
	collection[19]= prob_obj("hs16",2,5,0,
			  "min 100(x[2]-x[1]^2)^2+(1-x[1])^2\ns.t. x[1]+x[2]^2 >= 0\n     x[1]^2+x[2] >= 0\n     -2 <= x[1] <= 0.5, x[2] <= 1",
			  x -> 100(x[2]-x[1]^2)^2+(1-x[1])^2,
			  x -> [(-x[1]-x[2]^2),
				    (-x[1]^2-x[2]), (-x[1]-2), (x[1]-0.5), (x[2]-1)
				   ],
			  x -> zeros(0),
			  [[-2; 1]])
	collection[20]= prob_obj("hs17",2,5,0,
			  "min 100(x[2]-x[1]^2)^2+(1-x[1])^2\ns.t. x[2]^2-x[1] >= 0\n     x[1]^2-x[2] >= 0\n     -0.5 <= x[1] <= 0.5, x[2] <= 1",
			  x -> 100(x[2]-x[1]^2)^2+(1-x[1])^2,
			  x -> [(-x[2]^2+x[1]),
				    (-x[1]^2+x[2]), (-x[1]-0.5), (x[1]-0.5), (x[2]-1)
				   ],
			  x -> zeros(0),
			  [[-2; 1]])
	collection[21]= prob_obj("hs18",2,6,0,
			  "min 0.01x[1]^2+x[2]^2\ns.t. x[1]x[2]-25 >= 0\n     x[1]^2+x[2]^2-25 >= 0\n     2 <= x[1] <= 50\n     0 <= x[2] <= 50",
			  x -> 0.01x[1]^2+x[2]^2,
			  x -> [(-x[1]x[2]+25),
				    (-x[1]^2-x[2]^2+25), (-x[1]-2), (x[1]-50), (-x[2]), (x[2]-50)
				   ],
			  x -> zeros(0),
			  [[2; 2]])
	collection[22]= prob_obj("hs19",2,6,0,
			  "min (x[1]-10)^3+(x[2]-20)^3\ns.t. (x[1]-5)^2+(x[2]-5)^2 - 100 >= 0\n     (x[1]-6)^2+(x[2]-5)^2 - 82.81 <= 0\n     13 <= x[1] <= 100\n     0 <= x[2] <= 100",
			  x -> (x[1]-10)^3+(x[2]-20)^3,
			  x -> [(-(x[1]-5)^2-(x[2]-5)^2 + 100),
				    ((x[1]-6)^2+(x[2]-5)^2 - 82.81), 
				    (-x[1]+13), (x[1]-100), (-x[2]), (x[2]-100)
				   ],
			  x -> zeros(0),
			  [[20.1; 5.84]])
	collection[23]= prob_obj("hs20",2,5,0,
			  "min 100(x[2]-x[1]^2)^2+(1-x[1])^2\ns.t. x[1]+x[2]^2 >= 0\n     x[1]^2+x[2] >= 0\n     x[1]^2+x[2]^2-1 >= 0\n     -0.5 <= x[1] <= 0.5",
			  x -> 100(x[2]-x[1]^2)^2+(1-x[1])^2,
			  x -> [(-x[1]-x[2]^2),
				    (-x[1]^2-x[2]), 
				    (-x[1]^2-x[2]^2+1), (-x[1]-0.5), (x[1]-0.5)
				   ],
			  x -> zeros(0),
			  [[-2; 1]])
	collection[24]= prob_obj("hs21",2,5,0,
			  "min 0.01x[1]^2+x[2]^2-100\ns.t. -10x[1]+x[2]+10 <= 0\n     2 <= x[1] <= 50\n     -50 <= x[2] <= 50",
			  x -> 0.01x[1]^2+x[2]^2-100,
			  x -> [(-10x[1]+x[2]+10),
				    (-x[1]+2), (x[1]-50),
				    (-x[2]-50), (x[2]-50)
				   ],
			  x -> zeros(0),
			  [[-1; -1]])
	collection[25]= prob_obj("hs22",2,2,0,
			  "min (x[1]-2)^2+(x[2]-1)^2\ns.t. x[1]+x[2] <= 2\n     x[1]^2 - x[2] <= 0",
			  x -> (x[1]-2)^2+(x[2]-1)^2,
			  x -> [(x[1]+x[2] - 2),
				    (x[1]^2 - x[2])
				   ],
			  x -> zeros(0),
			  [[-1; -1]])
	collection[26]= prob_obj("hs23",2,9,0,
			  "min x[1]^2+x[2]^2\ns.t. x[1]+x[2] - 1 >= 0\n     x[1]^2 + x[2]^2 - 1 >= 0\n     9x[1]^2+x[2]^2 - 9 >= 0\n     x[1]^2-x[2] >= 0\n     x[2]^2-x[1] >= 0\n     -50 <= x[i] <= 50, i=1,2",
			  x -> x[1]^2+x[2]^2,
			  x -> [(-x[1]-x[2] + 1),
				    (-x[1]^2 - x[2]^2 + 1),
				    (-9x[1]^2-x[2]^2 + 9),
					(-x[1]^2+x[2]),
					(-x[2]^2+x[1]),
					(-x[1]-50), (x[1]-50),
					(-x[2]-50), (x[2]-50)
				   ],
			  x -> zeros(0),
			  [[3; 1]])
	collection[27]= prob_obj("hs24",2,5,0,
		      "min x[2]^3((x[1]-3)^2-9)/(23sqrt(3))\ns.t. -x[1]/sqrt(3)+x[2] <= 0\n     -x[1]-sqrt(3)x[2] <= 0\n     x[1]+sqrt(3)x[2] <= 6\n     x[1], x[2] >= 0",
		      x -> x[2]^3*((x[1]-3)^2-9)/(23*sqrt(3)),
		      x -> [(-x[1]/sqrt(3)+x[2]),
			    (-x[1]-sqrt(3)x[2]),
			    (x[1]+sqrt(3)x[2] - 6),
			    -x[1], -x[2]
			   ],
		      x -> zeros(0),
			  [[1; 0.5]])	
	collection[28]= prob_obj("hs25",3,6,0,
		      "min sum_{i=1}^99(f_i(x))^2\n     f_i(x) = -0.01*i + exp(-(u[i]-x[2])^x[3]/x[1])\n     u[i] = 25 + (-50log(0.01*i))^(2/3)\ns.t. 0.1 <= x[1] <= 100\n     0.0 <= x[2] <= 25.6\n     0.0 <= x[3] <= 5",
		      x -> begin
					 ff = 0.0
					 for i = 1:99
						ui  = 25.0 + (-50.0*log(0.01*i))^(2.0/3.0)
						ff += (-0.01*i + exp(-((ui-x[2])^x[3])/(abs(x[1])+1.e-3)))^2
					 end
					 return 1.e+2*ff
				   end,
		      x -> [(-x[1]+0.1), (x[1]-100),
			    (-x[2]), (x[2]-25.6),
			    (-x[3]), (x[3]-5)
			   ],
		      x -> zeros(0),
			  [[100; 12.5; 3],[50;25;1.5]])	
	collection[29]= prob_obj("hs26",3,0,1,
		      "min (x[1]-x[2])^2+(x[2]-x[3])^2\ns.t. (1+x[2]^2)x[1] + x[3]^4 - 3 = 0",
		      x -> (x[1]-x[2])^2+(x[2]-x[3])^2,
		      x -> zeros(0),
		      x -> [((1+x[2]^2)x[1] + x[3]^4 - 3)],
			  [[-2.6; 2; 2]])	
	collection[30]= prob_obj("hs27",3,0,1,
		      "min 0.01(x[1]-1)^2+(x[2]-x[1]^2)^2\ns.t. x[1] + x[3]^2 + 1 = 0",
		      x -> 0.01(x[1]-1)^2+(x[2]-x[1]^2)^2,
		      x -> zeros(0),
		      x -> [(x[1] + x[3]^2 + 1)],
			  [[2; 2; 2]])	
	collection[31]= prob_obj("hs28",3,0,1,
		      "min (x[1]+x[2])^2+(x[2]+x[3])^2\ns.t. x[1] + 2x[2] + 3x[3] -1 = 0",
		      x -> (x[1]+x[2])^2+(x[2]+x[3])^2,
		      x -> zeros(0),
		      x -> [(x[1] + 2x[2] + 3x[3] -1)],
			  [[-4; 1; 1]])	
	collection[32]= prob_obj("hs29",3,1,0,
		      "min -x[1]x[2]x[3]\ns.t. x[1]^2 + 2x[2]^2 + 4x[3]^2 <= 48",
		      x -> -x[1]x[2]x[3],
		      x -> [(x[1]^2 + 2x[2]^2 + 4x[3]^2 - 48)],
		      x -> zeros(0),
			  [[1; 1; 1]])	
	collection[33]= prob_obj("hs30",3,7,0,
		      "min x[1]^2+x[2]^2+x[3]^2\ns.t. x[1]^2 + x[2]^2 >= 1\n       1 <= x[1] <= 10\n     -10 <= x[2] <= 10\n     -10 <= x[3] <= 10",
		      x -> x[1]^2+x[2]^2+x[3]^2,
		      x -> [(-x[1]^2 - x[2]^2 + 1),
					(1-x[1]), (x[1]-10),
					(-10-x[2]), (x[2]-10),
					(-10-x[3]), (x[3]-10)
				   ],
		      x -> zeros(0),
			  [[1; 1; 1]])	
	collection[34]= prob_obj("hs31",3,7,0,
		      "min 9x[1]^2+x[2]^2+9x[3]^2\ns.t. x[1]x[2] >= 1\n     -10 <= x[1] <= 10\n       1 <= x[2] <= 10\n     -10 <= x[3] <= 1",
		      x -> 9x[1]^2+x[2]^2+9x[3]^2,
		      x -> [(-x[1]x[2] + 1),
					(-10-x[1]), (x[1]-10),
					(1-x[2]), (x[2]-10),
					(-10-x[3]), (x[3]-1)
				   ],
		      x -> zeros(0),
			  [[1; 1; 1]])	
	collection[35]= prob_obj("hs32",3,4,1,
		      "min (x[1]+3x[2]+x[3])^2+4(x[1]-x[2])^2\ns.t. 6x[2] + 4x[3] - x[1]^3 >= 3\n     x[1]+x[2]+x[3] = 1\n     x[i] >= 0,  i=1,2,3",
		      x -> (x[1]+3x[2]+x[3])^2+4(x[1]-x[2])^2,
		      x -> [(-6x[2] - 4x[3] + x[1]^3 + 3),
					(-x[1]), (-x[2]), (-x[3])
				   ],
		      x -> [(x[1]+x[2]+x[3] - 1)],
			  [[.1; .7; .2]])	
	collection[36]= prob_obj("hs33",3,6,0,
		      "min (x[1]-1)(x[1]-2)(x[1]-3)+x[3]\ns.t. x[3]^2 - x[2]^2 - x[1]^2 >= 0\n     x[1]^2 + x[2]^2 + x[3]^2 >= 4\n     x[3] <= 5\n     x[i] >= 0,  i=1,2,3",
		      x -> (x[1]-1)*(x[1]-2)*(x[1]-3)+x[3],
		      x -> [(-x[3]^2 + x[2]^2 + x[1]^2),
					(-x[1]^2 - x[2]^2 - x[3]^2 + 4),
					(-x[1]^3), (-x[2]), (-x[3]), (x[3]-5)
				   ],
		      x -> zeros(0),
			  [[0; 0; 3]])	
	collection[50]= prob_obj("hs47",5,0,3,
		      "min (x[1]-x[2])^2+(x[2]-x[3])^3+(x[3]-x[4])^4+(x[4]-x[5])^4+1.e-6x'*x\ns.t. x[1]+x[2]^2+x[3]^3 = 3\n     x[2]-x[3]^2+x[4] = 1\n     x[1]x[5] = 1",
		      x -> (x[1]-x[2])^2+(x[2]-x[3])^3+(x[3]-x[4])^4+(x[4]-x[5])^4+1.e-6x'*x,
		      x -> zeros(0),
		      x -> [(x[1]+x[2]^2+x[3]^3 - 3),
					(x[2]-x[3]^2+x[4] - 1),
					(x[1]x[5] - 1)
				   ],
			  [[2; sqrt(2); -1; 2-sqrt(2); 0.5]])	
	collection[55]= prob_obj("hs52",5,0,3,
		      "min (4x[1]-x[2])^2+(x[2]+x[3]-2)^2+(x[4]-1)^2+(x[5]-1)^2\ns.t. x[1]+3x[2] = 0\n     x[3]+x[4]-2x[5] = 0\n     x[2]-x[5] = 0",
		      x -> (4x[1]-x[2])^2+(x[2]+x[3]-2)^2+(x[4]-1)^2+(x[5]-1)^2,
		      x -> zeros(0),
		      x -> [(x[1]+3x[2]),
					(x[3]+x[4]-2x[5]),
					(x[2]-x[5])
				   ],
			  [[2; 2; 2; 2; 2]])	

	##############################################
	function Base.show(p::prob_obj;ext=true)
		if ext 
			@printf("  ---------------------------------\n")
			@printf("  |           name    n   m   p   |\n")
			#           123  1234567890  123 123 123
			@printf("  ---------------------------------\n")
			@printf("  |     %10s  %3d %3d %3d   |\n",p.name,p.n,p.m,p.p)
			@printf("  ---------------------------------\n")
			@printf("  |                               |\n")
		end
		lines = split(p.desc,'\n')
		nl = length(lines)
		for i = 1:nl		
			@printf("  |\t%s\n",lines[i])
		end
		@printf("  |                               |\n")
		if ext
			@printf("  ---------------------------------\n")
		end
	end
	function Base.show(c::Dict{Int,prob_obj};pdesc=false,by=:id)
		@printf("  ---------------------------------\n")
		@printf("  | id        name    n   m   p   |\n")
		#           123  1234567890  123 123 123
		@printf("  ---------------------------------\n")
		if by == :id
			cord = sort(collect(c),by=x->x[1])
		elseif by == :n
			cord = sort(collect(c),by=x->x[2].n)
		elseif by == :m
			cord = sort(collect(c),by=x->x[2].m)
		elseif by == :p
			cord = sort(collect(c),by=x->x[2].p)
		else
			cord = sort(collect(c),by=x->x[2].name)
		end
		for (i,p) in cord 
		#for (i,p) in c
			@printf("  |%3d  %10s  %3d %3d %3d   |\n",i,p.name,p.n,p.m,p.p)
			if pdesc
				show(p;ext=false)
				@printf("  ---------------------------------\n")
			end
		end
		if ~pdesc
			@printf("  ---------------------------------\n")
		end
	end

end
nothing
